//
//  NLWeakMutableArray.m
//  NuDataProvider
//
//  Created by Reid Main on 2012-10-12.
//  Copyright 2012 NuLayer Inc. All rights reserved.
//

#import "NLWeakMutableArray.h"
#import "NLWeakReference.h"

@implementation NLWeakMutableArray
{
    NSMutableArray *_weakReferences;
}

- (id)init
{
    return [self initWithCapacity:0];
}

- (id)initWithCapacity:(NSUInteger)numItems
{
    if((self = [super init]) == nil)
        return nil;
    
    _weakReferences = [[NSMutableArray alloc] initWithCapacity:numItems];
    
    return self;
}

- (id)initWithObjects:(const id [])objects count:(NSUInteger)count
{
    if((self = [self initWithCapacity:count]) == nil)
        return nil;
    
    for(NSInteger i = 0; i < count; i++)
        if(objects[i] != nil)
            [self addObject:objects[i]];
    
    return self;
}

- (NSUInteger)count
{
    return [_weakReferences count];
}

- (id)objectAtIndex:(NSUInteger)index
{
    return [[_weakReferences objectAtIndex:index] referencedObject];
}

- (void)addObject:(id)object
{
    NLWeakReference *weakReference = [NLWeakReference weakReferenceWithObject:object];
    
    [_weakReferences addObject:weakReference];
}

- (void)insertObject:(id)object atIndex:(NSUInteger)index
{
    NLWeakReference *weakReference = [NLWeakReference weakReferenceWithObject:object];
    
    [_weakReferences insertObject:weakReference atIndex:index];
}

- (void)removeObjectAtIndex:(NSUInteger)index
{
    [_weakReferences removeObjectAtIndex:index];
}

- (void)removeLastObject
{
    [_weakReferences removeLastObject];
}

- (void)replaceObjectAtIndex:(NSUInteger)index withObject:(id)object
{
    NLWeakReference *weakReference = [NLWeakReference weakReferenceWithObject:object];
    
    [_weakReferences replaceObjectAtIndex:index withObject:weakReference];
}

@end