//
//  NLWeakMutableDictionary.m
//  NuDataProvider
//
//  Created by Reid Main on 2012-10-12.
//  Copyright 2012 NuLayer Inc. All rights reserved.
//

#import "NLWeakMutableDictionary.h"
#import "NLWeakReference.h"

@implementation NLWeakMutableDictionary
{
    NSMutableDictionary *_weakReferences;
}

- (id)init
{
    if((self = [super init]) == nil)
        return nil;
    
    _weakReferences = [[NSMutableDictionary alloc] init];
    
    return self;
}

- (id)initWithObjects:(NSArray *)objects forKeys:(NSArray *)keys
{
    if([objects count] != [keys count])
        return nil;
    
    if((self = [super init]) == nil)
        return nil;
    
    for(NSInteger i = 0; i < [objects count]; i++)
        if(objects[i] != nil && keys[i] != nil)
            [self setObject:objects[i] forKey:keys[i]];
    
    return self;
}

- (NSUInteger)count
{
    return [_weakReferences count];
}

- (id)objectForKey:(id)key
{
    NLWeakReference *weakReference = [_weakReferences objectForKey:key];
    
    id object = [weakReference referencedObject];
    
    if(object == nil && key != nil)
        [_weakReferences removeObjectForKey:key];
    
    return object;
}

- (NSEnumerator *)keyEnumerator
{
    // Enumerate over a copy because -objectForKey: modifies the dictionary which could cause an exception.
    return [[_weakReferences allKeys] objectEnumerator];
}

- (void)setObject:(id)object forKey:(id<NSCopying>)key
{
    NLWeakReference *weakReference = [NLWeakReference weakReferenceWithObject:object];
    
    [_weakReferences setObject:weakReference forKey:key];
}

- (void)removeObjectForKey:(id)key
{
    [_weakReferences removeObjectForKey:key];
}

@end