//
//  NLDataProvider.h
//  NLDataProvider
//
//  Created by Remy Demarest on 14/09/2010.
//  Copyright 2010 Nulayer Inc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <NLFoundation/NLFoundation.h>

extern NSString *const NLDataProviderHTTPHeadersOptions;
extern NSString *const NLDataProviderPreventEmptyCompletionOption; // NSNumber containing a boolean, if YES and the result of the request is empty the block isn't executed, default NO
extern NSString *const NLDataProviderReturnRawDataOption;          // NSNumber containing a boolean, if YES the returned resources of the request will be a raw NSData object, default NO

extern NSString *const NLDataProviderPreventDiscardingOption;      // NSNumber containing a boolean
extern NSString *const NLDataProviderIncludeBaseURLOption;         // NSNumber containing a boolean
extern NSString *const NLDataProviderRequestRunLoopModesOption;    // NSArray of NSString to execute requests
extern NSString *const NLDataProviderBaseURLExtraInfo;             // NSURL object representing the base URL of the returned resources
extern NSString *const NLDataProviderNSHTTPURLResponseExtraInfo;   // NSHTTPURLResponse object representing the response returned by an HTTP server, nil if the returned object was NOT an NSHTTPURLResponse instance
extern NSString *const NLDataProviderNSURLResponseExtraInfo;       // NSURLResponse object representing the response returned by a server
extern NSString *const NLDataProviderNSURLResponseErrorPayload;    // The parsed response from the server, only included if the status code indicates an error. This allows us to still return nil for the resources in completion blocks

extern NSString *const NLDataProviderDidFinishCompletionBlockNotification; // Sent after a block returned its data
extern NSString *const NLDataProviderDidCancelCompletionBlockNotification; // Sent after a block was cancelled
extern NSString *const NLDataProviderDidBeginCompletionBlockNotification;  // Sent after the response header has been received
extern NSString *const NLDataProviderDidUpdateCompletionBlockNotification; // Sent every time the status of the block is updated (more data arrived)

extern NSString *const NLDataProviderDownloadedSizeUserInfoKey;
extern NSString *const NLDataProviderResourceExpectedSizeUserInfoKey;

extern NSString *const NLDataProviderErrorDomain;

enum _NLDataProviderError
{
    NLDataProviderInvalidResponseError    = -2,
    NLDataProviderCouldNotEncodeDataError = -1,
    NLDataProviderContentNotModifiedError = 304,
    NLDataProviderContentUnavailableError = 404,
};

@protocol NLDataProviderRequestDelegate <NSObject>
@optional
- (void)requestObserverDidBegin:(id)observer;
- (void)requestObserver:(id)observer didDownloadContentOfSize:(long long)downloadedSize expectedContentSize:(long long)expectedContentSize;
- (void)requestObserverDidCancel:(id)observer;
- (void)requestObserverDidFinish:(id)observer;
@end

@protocol NLDataProvider <NSObject>

@property(nonatomic, copy, readonly) NSURL *baseURL;
- (NSURL *)requestURLForResourceURL:(NSURL *)resourceURL;

// returns an opaque object if the request is taken care of
// returns nil if the request couldn't be managed
- (id)loadResourcesForURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

- (id)createResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

- (id)updateResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

- (id)deleteResourceForURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

// Observer is the opaque object returned by -loadResourcesForURL:options:completion:
- (void)removeCompletionBlock:(id)observer;

@optional

- (id)loadResourcesForURL:(NSURL *)resourceURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

- (id)createResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

- (id)updateResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

- (id)deleteResourceForURL:(NSURL *)resourceURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

@end

extern NSString *const NLDataParserNSStringEncodingOption;  // NSNumber containing an NSStringEncoding
extern NSString *const NLDataParserIANACharSetNameOption;   // NSString
extern NSString *const NLDataParserMIMETypeOption;          // NSString
extern NSString *const NLDataParserSuggestedFilenameOption; // NSString

extern NSString *const NLDataParserErrorDomain;

enum NLDataParserError
{
    NLDataParserErrorUnrecognizedTypeError = -1,
    NLDataParserErrorParseError            = -2,
};
typedef NSInteger NLDataParserError;

@protocol NLDataParser <NSObject>

- (BOOL)canParseMIMEType:(NSString *)MIMEType;
- (BOOL)canParsePathExtension:(NSString *)extension;
- (id)parsedObjectForData:(NSData *)data withURL:(NSURL *)aURL options:(NSDictionary *)options error:(NSError **)error;

@end

@protocol NLDataEncoder <NSObject>

- (NLFormEncoding *)encodedDataForObject:(id)objectToEncode withURL:(NSURL *)aURL options:(NSDictionary *)options error:(NSError **)error;

@end
