//
//  NLDataProvider.m
//  NLDataProvider
//
//  Created by Remy Demarest on 14/09/2010.
//  Copyright 2010 Nulayer Inc. All rights reserved.
//

#import "NLDataProvider.h"

NSString *const NLDataProviderEntityTagOption              = @"NLDataProviderEntityTagOption";
NSString *const NLDataProviderLastModifiedDateOption       = @"NLDataProviderLastModifiedDateOption";
NSString *const NLDataProviderHTTPHeadersOptions           = @"NLDataProviderHTTPHeadersOptions";
NSString *const NLDataProviderPreventEmptyCompletionOption = @"NLDataProviderPreventEmptyCompletionOption";
NSString *const NLDataProviderReturnRawDataOption          = @"NLDataProviderReturnRawDataOption";

NSString *const NLDataProviderPreventDiscardingOption      = @"NLDataProviderPreventDiscardingOption";
NSString *const NLDataProviderIncludeBaseURLOption         = @"NLDataProviderIncludeBaseURLOption";
NSString *const NLDataProviderRequestRunLoopModesOption    = @"NLDataProviderRequestRunLoopModesOption";
NSString *const NLDataProviderBaseURLExtraInfo             = @"NLDataProviderBaseURLExtraInfo";
NSString *const NLDataProviderNSHTTPURLResponseExtraInfo   = @"NLDataProviderNSHTTPURLResponseExtraInfo";
NSString *const NLDataProviderNSURLResponseExtraInfo       = @"NLDataProviderNSURLResponseExtraInfo";
NSString *const NLDataProviderNSURLResponseErrorPayload    = @"NLDataProviderNSURLResponseErrorPayload";

NSString *const NLDataProviderErrorDomain                  = @"com.nulayer.dataProvider.errorDomain";

NSString *const NLDataParserNSStringEncodingOption         = @"NLDataParserNSStringEncodingOption";
NSString *const NLDataParserIANACharSetNameOption          = @"NLDataParserIANACharSetNameOption";
NSString *const NLDataParserMIMETypeOption                 = @"NLDataParserMIMETypeOption";
NSString *const NLDataParserSuggestedFilenameOption        = @"NLDataParserSuggestedFilenameOption";

NSString *const NLDataParserErrorDomain                    = @"com.nulayer.dataParser.errorDomain";
NSString *const NLDataEncoderErrorDomain                   = @"com.nulayer.dataEncoder.errorDomain";

NSString *const NLDataProviderDidFinishCompletionBlockNotification = @"NLDataProviderDidFinishCompletionBlockNotification";
NSString *const NLDataProviderDidCancelCompletionBlockNotification = @"NLDataProviderDidCancelCompletionBlockNotification";
NSString *const NLDataProviderDidBeginCompletionBlockNotification  = @"NLDataProviderDidBeginCompletionBlockNotification";
NSString *const NLDataProviderDidUpdateCompletionBlockNotification = @"NLDataProviderDidUpdateCompletionBlockNotification";

NSString *const NLDataProviderDownloadedSizeUserInfoKey            = @"NLDataProviderDownloadedSizeUserInfoKey";
NSString *const NLDataProviderResourceExpectedSizeUserInfoKey      = @"NLDataProviderResourceExpectedSizeUserInfoKey";

@interface NSObject (NLDataProviderConformance)
- (id)loadResourcesForURL:(NSURL *)resourceURL options:(NSDictionary *)options completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
- (id)createResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
- (id)updateResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
- (id)deleteResourceForURL:(NSURL *)resourceURL options:(NSDictionary *)options completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
@end

@interface NLDataProviderRequestDelegateNotifier : NSObject <NLDataProviderRequestDelegate>
+ (NLDataProviderRequestDelegateNotifier *)defaultNotifier;
@end

@interface NSObject ()

- (id)loadResourcesForURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

- (id)createResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

- (id)updateResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

- (id)deleteResourceForURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

@end

@implementation NSObject (NLDataProvider)

- (id)loadResourcesForURL:(NSURL *)resourceURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
{
    return [self loadResourcesForURL:resourceURL options:options delegate:[NLDataProviderRequestDelegateNotifier defaultNotifier] resourceParser:parser completion:completion];
}

- (id)createResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
{
    return [self createResource:resources forURL:resourceURL options:options delegate:[NLDataProviderRequestDelegateNotifier defaultNotifier] resourceParser:parser completion:completion];
}

- (id)updateResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
{
    return [self updateResource:resources forURL:resourceURL options:options delegate:[NLDataProviderRequestDelegateNotifier defaultNotifier] resourceParser:parser completion:completion];
}

- (id)deleteResourceForURL:(NSURL *)resourceURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
{
    return [self deleteResourceForURL:resourceURL options:options delegate:[NLDataProviderRequestDelegateNotifier defaultNotifier] resourceParser:parser completion:completion];
}

@end

@implementation NLDataProviderRequestDelegateNotifier

+ (NLDataProviderRequestDelegateNotifier *)defaultNotifier;
{
    static NLDataProviderRequestDelegateNotifier *defaultNotifier = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        defaultNotifier = [[NLDataProviderRequestDelegateNotifier alloc] init];
    });
    
    return defaultNotifier;
}

- (void)requestObserverDidBegin:(id)observer;
{
    [[NSNotificationCenter defaultCenter] postNotificationName:NLDataProviderDidBeginCompletionBlockNotification object:observer];
}

- (void)requestObserver:(id)observer didDownloadContentOfSize:(long long)downloadedSize expectedContentSize:(long long)expectedContentSize;
{
    [[NSNotificationCenter defaultCenter] postNotificationName:NLDataProviderDidUpdateCompletionBlockNotification object:observer userInfo:
     [NSDictionary dictionaryWithObjectsAndKeys:
      [NSNumber numberWithLongLong:downloadedSize], NLDataProviderDownloadedSizeUserInfoKey,
      [NSNumber numberWithLongLong:expectedContentSize], NLDataProviderResourceExpectedSizeUserInfoKey,
      nil]];
}

- (void)requestObserverDidCancel:(id)observer;
{
    [[NSNotificationCenter defaultCenter] postNotificationName:NLDataProviderDidCancelCompletionBlockNotification object:observer];
}

- (void)requestObserverDidFinish:(id)observer;
{
    [[NSNotificationCenter defaultCenter] postNotificationName:NLDataProviderDidFinishCompletionBlockNotification object:observer];
}

@end
