//
//  NLHTTPRequestOperation.h
//  NLDataProvider
//
//  Created by Remy Demarest on 14/09/2010.
//  Copyright 2010 Nulayer Inc. All rights reserved.
//

#import <Foundation/Foundation.h>

extern NSString *const NLDataProviderContentTypeOption;     // NSString - application/x-www-form-urlencoded or multipart/form-data; boundary=
extern NSString *const NLDataProviderBodyOption;            // NSData or NSInputString, only valid for POST or PUT method
extern NSString *const NLDataProviderQueryParametersOption; // NSDictionary
extern NSString *const NLDataProviderDataParserOption;      // id<NLDataParser> Data Parser object to be used in priority to parse the returned data of the request
extern NSString *const NLHTTPRequestMethodOption;           // NSString - GET, POST, PUT or DELETE

@protocol NLHTTPRequestOperationDelegate;

@interface NLHTTPRequestOperation : NSOperation
{
@private
    id<NLHTTPRequestOperationDelegate> _delegate;
    void (^_requestCompletionBlock)(NSData *resources, NSHTTPURLResponse *response, NSError *error);
    
    NSURLConnection   *_connection;
    NSHTTPURLResponse *_response;
    NSMutableData     *_responseData;
    NSURL             *_requestURL;
    NSDictionary      *_requestOptions;
    NSTimeInterval     _timeoutInterval;
    
    struct
    {
        unsigned int isFinished:1;
        unsigned int isExecuting:1;
        unsigned int isCancelled:1;
        unsigned int delegateDidBegin:1;
        unsigned int delegateDidCancel:1;
        unsigned int delegateDidDownload:1;
        unsigned int delegateDidFinish:1;
    } _requestFlags;
}

+ (id)requestOperationWithURL:(NSURL *)aURL options:(NSDictionary *)options completion:(void (^)(NSData *resources, NSHTTPURLResponse *response, NSError *error))completion;
- (id)initWithURL:(NSURL *)aURL options:(NSDictionary *)options completion:(void (^)(NSData *resources, NSHTTPURLResponse *response, NSError *error))completion;

@property(nonatomic, readonly, copy) void (^requestCompletionBlock)(NSData *resources, NSHTTPURLResponse *response, NSError *error);
@property(nonatomic, weak)           id<NLHTTPRequestOperationDelegate> delegate;

@property(nonatomic, readonly, copy) NSURL        *requestURL;
@property(nonatomic, readonly, copy) NSDictionary *requestOptions;

// Gives the chance for subclasses to configure the request before sending
- (void)configureRequest:(NSMutableURLRequest *)request;

@property(nonatomic) NSTimeInterval timeoutInterval;

@end

@protocol NLHTTPRequestOperationDelegate <NSObject>
@optional

- (void)requestOperationDidBegin:(NLHTTPRequestOperation *)operation;
- (void)requestOperation:(NLHTTPRequestOperation *)operation didDownloadContentOfSize:(long long)downloaded expectedSize:(long long)expected;
- (void)requestOperationDidCancel:(NLHTTPRequestOperation *)operation;
- (void)requestOperationDidFinish:(NLHTTPRequestOperation *)operation;

@end
