//
//  NLRemoteDataProvider.h
//  NLDataProvider
//
//  Created by Remy Demarest on 14/09/2010.
//  Copyright 2010 Nulayer Inc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <NuDataProvider/NLDataProvider.h>

@class NLJSONDataParser;

@protocol NLRemoteDataProviderDelegate;

@interface NLRemoteDataProvider : NSObject <NLDataProvider>
{
@private
    id<NLRemoteDataProviderDelegate>  _delegate;
    NSArray                          *_dataParsers;
    NSArray                          *_dataEncoders;
    NSURL                            *_baseURL;
    NSOperationQueue                 *_operationQueue;
    NSMutableSet                     *_currentOperations;
    
    struct
    {
        unsigned int delegateRequestURLForResourceURL:1;
        unsigned int delegateDidBeginRequest:1;
        unsigned int delegateDidFinishRequest:1;
        unsigned int delegateDidCancelRequest:1;
        unsigned int delegateDidFinishAllRequests:1;
        unsigned int delegateShouldBeginRequest:1;
        unsigned int delegateWillReturnResources:1;
    } _remoteDataProviderFlags;
}

- (id)initWithRequestFinalizeQueue:(dispatch_queue_t)queue;

@property(nonatomic, weak) id<NLRemoteDataProviderDelegate> delegate;

@property(nonatomic, copy) NSURL *baseURL;


@property(nonatomic, copy) NSString *userAgent;

@property(nonatomic, copy) NSArray *dataParsers;
@property(nonatomic, copy) NSArray *dataEncoders;

- (NSURL *)requestURLForRelativePath:(NSString *)aPath;

@end

@protocol NLRemoteDataProviderDelegate <NSObject>
@optional
- (NSURL *)remoteDataProvider:(NLRemoteDataProvider *)sender requestURLForResourceURL:(NSURL *)resourceURL;
- (void)remoteDataProvider:(NLRemoteDataProvider *)sender didBeginRequestForURL:(NSURL *)URLRequest;
- (void)remoteDataProvider:(NLRemoteDataProvider *)sender didFinishRequestForURL:(NSURL *)URLRequest withResponse:(NSHTTPURLResponse *)response error:(NSError *)error;
- (void)remoteDataProvider:(NLRemoteDataProvider *)sender didCancelRequestForURL:(NSURL *)URLRequest;
- (void)remoteDataProviderDidFinishAllRequests:(NLRemoteDataProvider *)sender;

- (BOOL)remoteDataProvider:(NLRemoteDataProvider *)sender shouldBeginRequestForResourceURL:(NSURL *)resourceURL options:(NSDictionary *)options completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completionBlock;
- (void)remoteDataProvider:(NLRemoteDataProvider *)sender willReturnResources:(id)resources forResourceURL:(NSURL *)resourceURL withExtraInfo:(NSDictionary *)extraInfo error:(NSError *)error;

@end

