//
//  NLRemoteDataProvider.m
//  NLDataProvider
//
//  Created by Remy Demarest on 14/09/2010.
//  Copyright 2010 Nulayer Inc. All rights reserved.
//

#import "NLRemoteDataProvider.h"
#import "NLWeakMutableArray.h"
#import <NLFoundation/NLFoundation.h>
#import "NLHTTPRequestOperation.h"

#if __has_feature(objc_arc) && OS_OBJECT_USE_OBJC_RETAIN_RELEASE
#define NL_dispatch_retain(obj) (void)({})
#define NL_dispatch_release(obj) (void)({})
#else
#define NL_dispatch_retain(obj) dispatch_retain(obj)
#define NL_dispatch_release(obj) dispatch_release(obj)
#endif

static NSString *const NLHTTPRequestGETMethod    = @"GET";
static NSString *const NLHTTPRequestPOSTMethod   = @"POST";
static NSString *const NLHTTPRequestPUTMethod    = @"PUT";
static NSString *const NLHTTPRequestDELETEMethod = @"DELETE";

static NSString *const NLHTTPRequestContentType = @"Content-Type";
static NSString *const NLHTTPRequestBody        = @"Body";

@interface NLDataCompletionHandler : NSObject <NLHTTPRequestOperationDelegate>

+ (id)handlerWithDelegate:(NLRemoteDataProvider *)delegate;

@property(nonatomic, weak) NLRemoteDataProvider *delegateProvider;
@property(nonatomic, readonly, copy) NSArray *completionBlocks;

@property(nonatomic) NLHTTPRequestOperation *operation;

- (id)addResourceURL:(NSURL *)aURL delegate:(id<NLDataProviderRequestDelegate>)requestDelegate forCompletionBlock:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))block;
- (void)removeCompletionBlock:(id)observer;
- (void)returnResourcesToBlocks:(id)resources extraInfo:(NSDictionary *)extraInfo error:(NSError *)error forHandler:(NLRemoteDataProvider *)handler;

@end

@interface NLDataCompletionBlockHandler : NSObject
+ (id)blockHandlerWithOwner:(NLDataCompletionHandler *)owner resourceURL:(NSURL *)aURL delegate:(id<NLDataProviderRequestDelegate>)requestDelegate block:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))block;
@property(nonatomic, weak) id<NLDataProviderRequestDelegate> requestDelegate;
@property(nonatomic, weak) NLDataCompletionHandler *owner;
@property(nonatomic, copy)   NSURL *resourceURL;
@property(nonatomic, copy)   void(^completionBlock)(id resources, NSDictionary *extraInfo, NSError *error);
@property(nonatomic) NSOperationQueue *queue;
- (void)cancel;
@end

@interface NLRemoteDataProvider ()
{
    dispatch_queue_t _requestFinalizingQueue;
}

- (void)NL_removeHandlerForOperation:(NLHTTPRequestOperation *)operation;
- (void)NL_removeHandler:(NLDataCompletionHandler *)handler;
- (NLDataCompletionHandler *)NL_handlerForURL:(NSURL *)aURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser;
- (NLDataCompletionHandler *)NL_createHandlerForURL:(NSURL *)aURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser;
- (id)NL_parsedResourcesForData:(NSData *)data forURL:(NSURL *)aURL options:(NSDictionary *)options URLResponse:(NSHTTPURLResponse *)response error:(NSError **)outError;
- (NLFormEncoding *)NL_encodedDataForObject:(id)objectToEncode withURL:(NSURL *)aURL options:(NSDictionary *)options error:(NSError **)error;
- (NSDictionary *)NL_optionsForCreateOrUpdateRequestWithResource:(id)resourceObject URL:(NSURL *)resourceURL options:(NSDictionary *)options error:(NSError **)error;
- (id)NL_setupCreateOrUpdateResourceRequestForResource:(id)resources withURL:(NSURL *)resourceURL HTTPMethod:(NSString *)HTTPMethod options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)requestDelegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void (^)(id, NSDictionary *, NSError *))completion;

- (void)NL_notifyDelegateDidFinishAllRequests;
- (void)NL_notifyDelegateDidCancelRequestWithURL:(NSURL *)aURL;
- (void)NL_notifyDelegateDidFinishRequestWithURL:(NSURL *)aURL response:(NSHTTPURLResponse *)response error:(NSError *)error;
- (void)NL_notifyDelegateDidBeginRequestWithURL:(NSURL *)aURL;
- (BOOL)NL_notifyDelegateShouldBeginRequestForResourceURL:(NSURL *)resourceURL options:(NSDictionary *)options completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completionBlock;
- (void)NL_notifyDelegateWillReturnResources:(id)resources forResourceURL:(NSURL *)resourceURL withExtraInfo:(NSDictionary *)extraInfo error:(NSError *)error;

- (id)NL_setupResourceLoadingForURL:(NSURL *)resourceURL HTTPMethod:(NSString *)HTTPMethod options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)requestDelegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;

- (NSDictionary *)NL_extraInfoForURL:(NSURL *)resourcePath resources:(id)resources error:(NSError *)error response:(NSHTTPURLResponse *)response options:(NSDictionary *)options;

@end


@implementation NLRemoteDataProvider
@synthesize userAgent, baseURL = _baseURL, dataParsers = _dataParsers, dataEncoders = _dataEncoders;

- (id)init
{
    return nil;
}

- (id)initWithRequestFinalizeQueue:(dispatch_queue_t)queue;
{
    if((self = [super init]))
    {
        _operationQueue    = [[NSOperationQueue alloc] init];
        [_operationQueue setMaxConcurrentOperationCount:10];
        
        _currentOperations = [[NSMutableSet alloc] init];
        
        NL_dispatch_retain(queue);
        _requestFinalizingQueue = queue;
    }
    return self;
}

- (void)dealloc
{
    if(_requestFinalizingQueue != NULL) NL_dispatch_release(_requestFinalizingQueue), _requestFinalizingQueue = nil;
}

#pragma mark -
#pragma mark Delegation

- (id<NLRemoteDataProviderDelegate>)delegate { return _delegate; }
- (void)setDelegate:(id<NLRemoteDataProviderDelegate>)value
{
    if(_delegate != value)
    {
        _delegate = value;
        
        _remoteDataProviderFlags.delegateRequestURLForResourceURL = [_delegate respondsToSelector:@selector(remoteDataProvider:requestURLForResourceURL:)];
        _remoteDataProviderFlags.delegateDidBeginRequest          = [_delegate respondsToSelector:@selector(remoteDataProvider:didBeginRequestForURL:)];
        _remoteDataProviderFlags.delegateDidFinishRequest         = [_delegate respondsToSelector:@selector(remoteDataProvider:didFinishRequestForURL:withResponse:error:)];
        _remoteDataProviderFlags.delegateDidCancelRequest         = [_delegate respondsToSelector:@selector(remoteDataProvider:didCancelRequestForURL:)];
        _remoteDataProviderFlags.delegateDidFinishAllRequests     = [_delegate respondsToSelector:@selector(remoteDataProviderDidFinishAllRequests:)];
        _remoteDataProviderFlags.delegateShouldBeginRequest       = [_delegate respondsToSelector:@selector(remoteDataProvider:shouldBeginRequestForResourceURL:options:completion:)];
        _remoteDataProviderFlags.delegateWillReturnResources      = [_delegate respondsToSelector:@selector(remoteDataProvider:willReturnResources:forResourceURL:withExtraInfo:error:)];
    }
}

- (void)NL_notifyDelegateDidFinishAllRequests;
{
    if(_remoteDataProviderFlags.delegateDidFinishAllRequests)
        [[self delegate] remoteDataProviderDidFinishAllRequests:self];
}

- (void)NL_notifyDelegateDidCancelRequestWithURL:(NSURL *)aURL;
{
    if(_remoteDataProviderFlags.delegateDidCancelRequest)
        [[self delegate] remoteDataProvider:self didCancelRequestForURL:aURL];
}

- (void)NL_notifyDelegateDidFinishRequestWithURL:(NSURL *)aURL response:(NSHTTPURLResponse *)response error:(NSError *)error;
{
    if(_remoteDataProviderFlags.delegateDidFinishRequest)
        [[self delegate] remoteDataProvider:self didFinishRequestForURL:aURL withResponse:response error:error];
}

- (void)NL_notifyDelegateDidBeginRequestWithURL:(NSURL *)aURL;
{
    if(_remoteDataProviderFlags.delegateDidBeginRequest)
        [[self delegate] remoteDataProvider:self didBeginRequestForURL:aURL];
}

- (BOOL)NL_notifyDelegateShouldBeginRequestForResourceURL:(NSURL *)resourceURL options:(NSDictionary *)options completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completionBlock;
{
    if(_remoteDataProviderFlags.delegateShouldBeginRequest)
        return [[self delegate] remoteDataProvider:self shouldBeginRequestForResourceURL:resourceURL options:options completion:completionBlock];
    
    return YES;
}

- (void)NL_notifyDelegateWillReturnResources:(id)resources forResourceURL:(NSURL *)resourceURL withExtraInfo:(NSDictionary *)extraInfo error:(NSError *)error;
{
    if(_remoteDataProviderFlags.delegateWillReturnResources)
        [[self delegate] remoteDataProvider:self willReturnResources:resources forResourceURL:resourceURL withExtraInfo:extraInfo error:error];
}

#pragma mark -
#pragma mark URL

- (NSURL *)requestURLForRelativePath:(NSString *)aPath
{
    NSParameterAssert(aPath != nil);
    
    NSAssert(_baseURL != nil, @"The remoteBaseURL is unspecified.");
    
    return [_baseURL URLByAppendingPathComponent:aPath];
}

- (NSURL *)requestURLForResourceURL:(NSURL *)resourceURL;
{
    if(_remoteDataProviderFlags.delegateRequestURLForResourceURL)
        resourceURL = [[self delegate] remoteDataProvider:self requestURLForResourceURL:resourceURL];
    
    return resourceURL;
}

#pragma mark -
#pragma mark Operation management

- (NLDataCompletionHandler *)NL_handlerForURL:(NSURL *)aURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser;
{
    NLDataCompletionHandler *ret = nil;
    
    @synchronized(_currentOperations)
    {
        ret = [[_currentOperations objectsPassingTest:
               ^ BOOL (NLDataCompletionHandler *obj, BOOL *stop)
               {
                   NLHTTPRequestOperation *operation = [obj operation];
                   
                   if([[operation requestURL] isEqual:aURL] &&
                      ([operation requestOptions] == options ||
                       [options isEqualToDictionary:[operation requestOptions]]))
                   {
                       *stop = YES;
                       return YES;
                   }
                   
                   return NO;
               }] anyObject];
        
        if(ret == nil || [[ret operation] isCancelled] || [[ret operation] isFinished])
            ret = [self NL_createHandlerForURL:aURL options:options resourceParser:parser];
    }
    
    return ret;
}

- (NLDataCompletionHandler *)NL_createHandlerForURL:(NSURL *)aURL options:(NSDictionary *)options resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser;
{
    NLDataCompletionHandler *ret = [NLDataCompletionHandler handlerWithDelegate:self];
    [_currentOperations addObject:ret];
    
    NLHTTPRequestOperation *operation =
    [NLHTTPRequestOperation requestOperationWithURL:aURL
                                            options:options
                                         completion:
     ^(NSData *resources, NSHTTPURLResponse *response, NSError *error)
     {
         [self NL_notifyDelegateDidFinishRequestWithURL:[[ret operation] requestURL] response:response error:error];
         
         dispatch_async(_requestFinalizingQueue, ^(void) {
             id parsed = nil;
             
             NSError *deeperError = error;
             
             if([resources length] > 0) parsed = [self NL_parsedResourcesForData:resources
                                                                          forURL:aURL
                                                                         options:options
                                                                     URLResponse:response
                                                                           error:&deeperError];
             /*
             if(parsed == nil)
                 NSLog(@"Will return error: %@, %@\n"
                       @"requestURL:  %@\n"
                       @"options:     %@", error, [error userInfo], aURL, options);
              */
             
             NSDictionary *extraInfo = [self NL_extraInfoForURL:aURL resources:parsed error:deeperError response:response options:options];
             
             if(parser != nil) parsed = parser(parsed, extraInfo, &deeperError);
             
             // allow the response data to be included in the extraInfo, but return nil resources if the status indicates an error
             NSInteger statusCode = [response statusCode];
             if(!(200 <= statusCode && statusCode < 400))
                 parsed = nil;
             
             [ret returnResourcesToBlocks:parsed extraInfo:extraInfo error:deeperError forHandler:self];
         });
         
         [self NL_removeHandler:ret];
     }];
    
    [ret setOperation:operation];
    
    [_operationQueue addOperation:operation];
    
    [self NL_notifyDelegateDidBeginRequestWithURL:[operation requestURL]];
    
    return ret;
}

- (void)NL_removeHandlerForOperation:(NLHTTPRequestOperation *)operation;
{
    NLDataCompletionHandler *found = [[_currentOperations objectsPassingTest:
                                       ^ BOOL (NLDataCompletionHandler *obj, BOOL *stop)
                                       {
                                           if([obj operation] == operation)
                                           {
                                               *stop = YES;
                                               return YES;
                                           }
                                           
                                           return NO;
                                       }] anyObject];
    
    [self NL_removeHandler:found];
}

- (void)NL_removeHandler:(NLDataCompletionHandler *)handler;
{
    @synchronized(_currentOperations)
    {
        [_currentOperations removeObject:handler];
    }
}

- (id<NLDataParser>)NL_dataParserForData:(NSData *)data forURL:(NSURL *)aURL options:(NSDictionary *)options URLResponse:(NSHTTPURLResponse *)response error:(NSError **)outError;
{
    id<NLDataParser> parser = [options objectForKey:NLDataProviderDataParserOption];
    
    if(parser != nil &&
       ([parser canParseMIMEType:[response MIMEType]]              ||
        [parser canParsePathExtension:[[aURL path] pathExtension]] ||
        [parser canParsePathExtension:[[response suggestedFilename] pathExtension]]))
        return parser;
    
    for(id<NLDataParser> parser in _dataParsers)
        if([parser canParseMIMEType:[response MIMEType]]              ||
           [parser canParsePathExtension:[[aURL path] pathExtension]] ||
           [parser canParsePathExtension:[[response suggestedFilename] pathExtension]])
            return parser;
    
    return nil;
}

- (id)NL_parsedResourcesForData:(NSData *)data forURL:(NSURL *)aURL options:(NSDictionary *)options URLResponse:(NSHTTPURLResponse *)response error:(NSError **)outError;
{
    if([[options objectForKey:NLDataProviderReturnRawDataOption] boolValue]) return data;
    
    id ret = nil;
    
    id<NLDataParser> parser = [self NL_dataParserForData:data forURL:aURL options:options URLResponse:response error:outError];
    
    if(parser != nil)
    {
        NSMutableDictionary *allOptions = [NSMutableDictionary dictionary];
        
        void(^addEntry)(id, id) = ^(id object, id key) { if(object != nil) [allOptions setObject:object forKey:key]; };
        
        addEntry([response textEncodingName] , NLDataParserIANACharSetNameOption  );
        addEntry([response MIMEType]         , NLDataParserMIMETypeOption         );
        addEntry([response suggestedFilename], NLDataParserSuggestedFilenameOption);
        
        [allOptions addEntriesFromDictionary:options];
        
        ret = [parser parsedObjectForData:data withURL:aURL options:allOptions error:outError];
    }
    
    return ret;
}

- (NLFormEncoding *)NL_encodedDataForObject:(id)objectToEncode withURL:(NSURL *)aURL options:(NSDictionary *)options error:(NSError **)error;
{
    NSError *lastError = nil;
    
    NLFormEncoding *ret = nil;
    
    if([objectToEncode isKindOfClass:[NSData class]])
        ret = [NLFormEncoding newEncodingForBody:objectToEncode];
    else for(id<NLDataEncoder> encoder in [self dataEncoders])
    {
        ret = [encoder encodedDataForObject:objectToEncode withURL:aURL options:options error:(error != NULL ? &lastError : NULL)];
        
        if(ret != nil) break;
    }
    
    if(ret == nil && error != NULL)
    {
        *error = [NSError errorWithDomain:NLDataProviderErrorDomain code:NLDataProviderCouldNotEncodeDataError userInfo:
                  [NSDictionary dictionaryWithObjectsAndKeys:
                   lastError, NSUnderlyingErrorKey,
                   nil]];
    }
    
    return ret;
}

- (NSDictionary *)NL_extraInfoForURL:(NSURL *)aURL resources:(id)resources error:(NSError *)error response:(NSHTTPURLResponse *)response options:(NSDictionary *)options;
{
    NSMutableDictionary *ret = [NSMutableDictionary dictionary];
    
    if([[options objectForKey:NLDataProviderIncludeBaseURLOption] boolValue])
        [ret setObject:[self baseURL] forKey:NLDataProviderBaseURLExtraInfo];
    
    if(response != nil)
    {
        [ret setObject:response forKey:NLDataProviderNSURLResponseExtraInfo];
        
        
        NSInteger statusCode = [response statusCode];
        if(resources != nil && !(200 <= statusCode && statusCode != 304 && statusCode < 400))
            [ret setObject:resources forKey:NLDataProviderNSURLResponseErrorPayload];
        
        // FIXME: Needs to go away
        [ret addEntriesFromDictionary:[response dictionaryWithValuesForKeys:[NSArray arrayWithObjects:@"URL", @"expectedContentLength", @"MIMEType", @"textEncodingName", @"suggestedFilename", nil]]];
        
        if([response isKindOfClass:[NSHTTPURLResponse class]])
        {
            [ret setObject:response forKey:NLDataProviderNSHTTPURLResponseExtraInfo];
            
            // FIXME: Needs to go away
            [ret addEntriesFromDictionary:[response dictionaryWithValuesForKeys:[NSArray arrayWithObjects:@"allHeaderFields", @"statusCode", @"lastModifiedDate", @"reloadAfterDate", @"entityTag", nil]]];
        }
        
    }
    
    return ret;
}

static NSString *const NLRemoteDataProviderHandler  = @"NLRemoteDataProviderHandler";
static NSString *const NLRemoteDataProviderObserver = @"NLRemoteDataProviderObserver";

- (id)NL_setupResourceLoadingForURL:(NSURL *)resourceURL HTTPMethod:(NSString *)HTTPMethod options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)requestDelegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
{
    if(resourceURL == nil) return nil;
    
    NSMutableDictionary *allOptions = [NSMutableDictionary dictionaryWithCapacity:[options count] + 1];
    
    [allOptions setObject:HTTPMethod forKey:NLHTTPRequestMethodOption];
    
    [options enumerateKeysAndObjectsUsingBlock:
     ^(NSString *key, id obj, BOOL *stop)
     {
         if(userAgent != nil
            && [key isEqual:NLDataProviderHTTPHeadersOptions]
            && [obj objectForKey:@"User-Agent"] == nil)
         {
             NSMutableDictionary *temp = [obj mutableCopy];
             [temp setObject:userAgent forKey:@"User-Agent"];
             [temp removeObjectsForKeys:[temp allKeysForObject:[NSNull null]]];
             obj = [temp copy];
         }
         
         if(obj != [NSNull null]) [allOptions setObject:obj forKey:key];
     }];
    
    allOptions = [allOptions copy];
    
    if(![self NL_notifyDelegateShouldBeginRequestForResourceURL:resourceURL options:options completion:completion])
        return nil;
    
    NSURL *requestURL = [self requestURLForResourceURL:resourceURL];
    
    return [[self NL_handlerForURL:requestURL options:allOptions resourceParser:parser] addResourceURL:resourceURL delegate:requestDelegate forCompletionBlock:completion];
}

- (NSDictionary *)NL_optionsForCreateOrUpdateRequestWithResource:(id)resourceObject URL:(NSURL *)resourceURL options:(NSDictionary *)options error:(NSError **)error;
{
    NLFormEncoding *encodedData = [self NL_encodedDataForObject:resourceObject withURL:resourceURL options:options error:error];
    
    if(encodedData == nil) return nil;
    
    NSMutableDictionary *result = (options != nil ? [options mutableCopy] : [[NSMutableDictionary alloc] initWithCapacity:1]);
    
    if([encodedData contentType] != nil)
        [result setObject:[encodedData contentType] forKey:NLDataProviderContentTypeOption];
    
    [result setObject:[encodedData body] forKey:NLDataProviderBodyOption];
    
    NSDictionary *ret = [result copy];
    
    return ret;
}

- (id)NL_setupCreateOrUpdateResourceRequestForResource:(id)resources withURL:(NSURL *)resourceURL HTTPMethod:(NSString *)HTTPMethod options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)requestDelegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void (^)(id, NSDictionary *, NSError *))completion;
{
    NSError *error = nil;
    NSDictionary *allOptions = [self NL_optionsForCreateOrUpdateRequestWithResource:resources URL:resourceURL options:options error:&error];
    
    if(allOptions == nil)
    {
        completion(nil, nil, error);
        return nil;
    }
    
    return [self NL_setupResourceLoadingForURL:resourceURL HTTPMethod:HTTPMethod options:allOptions delegate:requestDelegate resourceParser:parser completion:completion];
}

#pragma mark -
#pragma mark NLDataProvider protocol conformance

- (id)loadResourcesForURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
{
    return [self NL_setupResourceLoadingForURL:resourceURL HTTPMethod:NLHTTPRequestGETMethod options:options delegate:delegate resourceParser:parser completion:completion];
}

- (id)createResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
{
    return [self NL_setupCreateOrUpdateResourceRequestForResource:resources withURL:resourceURL HTTPMethod:NLHTTPRequestPOSTMethod options:options delegate:delegate resourceParser:parser completion:completion];
}

- (id)updateResource:(id)resources forURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
{
    return [self NL_setupCreateOrUpdateResourceRequestForResource:resources withURL:resourceURL HTTPMethod:NLHTTPRequestPUTMethod options:options delegate:delegate resourceParser:parser completion:completion];
}

- (id)deleteResourceForURL:(NSURL *)resourceURL options:(NSDictionary *)options delegate:(id<NLDataProviderRequestDelegate>)delegate resourceParser:(id(^)(id resources, NSDictionary *extraInfo, NSError **error))parser completion:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))completion;
{
    return [self NL_setupResourceLoadingForURL:resourceURL HTTPMethod:NLHTTPRequestDELETEMethod options:options delegate:delegate resourceParser:parser completion:completion];
}

- (void)removeCompletionBlock:(id)observer;
{
    if(observer != nil && [observer isKindOfClass:[NLDataCompletionBlockHandler class]]) [observer cancel];
}

@end


@implementation NLDataCompletionHandler
{
    NLWeakMutableArray *_completionBlocks;
}

@synthesize operation = _operation, delegateProvider = _delegateProvider;

+ (id)handlerWithDelegate:(NLRemoteDataProvider *)delegate;
{
    NLDataCompletionHandler *ret = [[self alloc] init];
    [ret setDelegateProvider:delegate];
    return ret;
}

- (id)init;
{
    if((self = [super init]))
    {
        _completionBlocks = [[NLWeakMutableArray alloc] init];
    }
    return self;
}

- (void)dealloc
{
    [self setDelegateProvider:nil];
    [_operation setDelegate:nil];
    
    NSArray *blocks = _completionBlocks;
    _completionBlocks = nil;
    
    [blocks enumerateObjectsUsingBlock:
     ^(NLDataCompletionBlockHandler *blkHandler, NSUInteger idx, BOOL *stop)
     {
         [blkHandler setOwner:nil];
     }];
}

- (NSArray *)completionBlocks { return _completionBlocks; }

- (id)addResourceURL:(NSURL *)aURL delegate:(id<NLDataProviderRequestDelegate>)requestDelegate forCompletionBlock:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))block;
{
    NLDataCompletionBlockHandler *blkHandler = [NLDataCompletionBlockHandler blockHandlerWithOwner:self resourceURL:aURL delegate:requestDelegate block:block];
    
    [_completionBlocks addObject:blkHandler];
    
    return blkHandler;
}

- (void)removeCompletionBlock:(NLDataCompletionBlockHandler *)observer;
{
    if(observer == nil) return;
    
    __block BOOL observerWasFound = NO;
    
    @synchronized(self)
    {
        NSIndexSet *removedIndexes =
        [_completionBlocks indexesOfObjectsPassingTest:
         ^ BOOL (NLDataCompletionBlockHandler *blkHandler, NSUInteger idx, BOOL *stop)
         {
             if(blkHandler == observer)
             {
                 observerWasFound = YES;
                 return YES;
             }
             
             return blkHandler == nil;
         }];
        
        if(observerWasFound)
        {
            [observer setOwner:nil];
            [observer setCompletionBlock:nil];
        }
        
        [_completionBlocks removeObjectsAtIndexes:removedIndexes];
    }
    
    if(observerWasFound && [[observer requestDelegate] respondsToSelector:@selector(requestObserverDidCancel:)])
        [[observer requestDelegate] requestObserverDidCancel:observer];
    
    @synchronized(self)
    {
        if([_completionBlocks count] == 0) [_operation cancel];
    }
}

- (void)returnResourcesToBlocks:(id)resources extraInfo:(NSDictionary *)extraInfo error:(NSError *)error forHandler:(NLRemoteDataProvider *)handler;
{
    dispatch_group_t group  = dispatch_group_create();
    
    // When this method is reached, it's not possible to cancel blocks
    // Make a copy to prevent mutations and clean up the old block array
    NSArray *blocks = nil;
    
    @synchronized(self)
    {
        blocks = [_completionBlocks copy];
        RELEASE(_completionBlocks);
    }
    
    [blocks enumerateObjectsUsingBlock:
     ^(NLDataCompletionBlockHandler *blkHandler, NSUInteger idx, BOOL *stop)
     {
         if(blkHandler == nil) return;
         
         [blkHandler setOwner:nil];
         
         [[blkHandler queue] addOperationWithBlock:^{
             [handler NL_notifyDelegateWillReturnResources:resources forResourceURL:[blkHandler resourceURL] withExtraInfo:extraInfo error:error];
             
             void(^block)(id resources, NSDictionary *extraInfo, NSError *error) = [blkHandler completionBlock];
             if(block != nil) block(resources, extraInfo, error);
             
             if([[blkHandler requestDelegate] respondsToSelector:@selector(requestObserverDidFinish:)])
                 [[blkHandler requestDelegate] requestObserverDidFinish:blkHandler];
         }];
     }];
    
    dispatch_group_wait(group, DISPATCH_TIME_FOREVER);
    NL_dispatch_release(group);
    
}

- (void)setOperation:(NLHTTPRequestOperation *)operation
{
    if(_operation != operation)
    {
        if([_operation delegate] == self) [_operation setDelegate:nil];
        _operation = operation;
        [_operation setDelegate:self];
    }
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"<%@ %p requestURL: %@, options: %@>", [self class], self, [_operation requestURL], [_operation requestOptions]];
}

#pragma mark -
#pragma mark NLHTTPRequestOperationDelegate protocol conformance

- (void)requestOperationDidBegin:(NLHTTPRequestOperation *)operation
{
    [_completionBlocks enumerateObjectsUsingBlock:
     ^(NLDataCompletionBlockHandler *blkHandler, NSUInteger idx, BOOL *stop)
     {
         if(blkHandler == nil) return;
         
         [[blkHandler queue] addOperationWithBlock:^{
             if([[blkHandler requestDelegate] respondsToSelector:@selector(requestObserverDidBegin:)])
                 [[blkHandler requestDelegate] requestObserverDidBegin:blkHandler];
         }];
     }];
}

- (void)requestOperation:(NLHTTPRequestOperation *)operation didDownloadContentOfSize:(long long)downloaded expectedSize:(long long)expected
{
    [_completionBlocks enumerateObjectsUsingBlock:
     ^(NLDataCompletionBlockHandler *blkHandler, NSUInteger idx, BOOL *stop)
     {
         if(blkHandler == nil) return;
         
         [[blkHandler queue] addOperationWithBlock:^{
             if([[blkHandler requestDelegate] respondsToSelector:@selector(requestObserver:didDownloadContentOfSize:expectedContentSize:)])
                 [[blkHandler requestDelegate] requestObserver:blkHandler didDownloadContentOfSize:downloaded expectedContentSize:expected];
         }];         
     }];
}

- (void)requestOperationDidCancel:(NLHTTPRequestOperation *)operation
{
    [_completionBlocks enumerateObjectsUsingBlock:
     ^(NLDataCompletionBlockHandler *blkHandler, NSUInteger idx, BOOL *stop)
     {
         if(blkHandler == nil) return;
         
         [blkHandler setOwner:nil];
         
         [[blkHandler queue] addOperationWithBlock:^{
             if([[blkHandler requestDelegate] respondsToSelector:@selector(requestObserverDidCancel:)])
                 [[blkHandler requestDelegate] requestObserverDidCancel:blkHandler];
         }];
     }];
}

@end

@implementation NLDataCompletionBlockHandler
{
}

@synthesize owner, resourceURL, completionBlock, queue;

+ (id)blockHandlerWithOwner:(NLDataCompletionHandler *)owner resourceURL:(NSURL *)aURL delegate:(id<NLDataProviderRequestDelegate>)requestDelegate block:(void(^)(id resources, NSDictionary *extraInfo, NSError *error))block;
{
    NLDataCompletionBlockHandler *ret = [[self alloc] init];
    
    [ret setOwner:owner];
    [ret setResourceURL:aURL];
    [ret setCompletionBlock:block];
    [ret setRequestDelegate:requestDelegate];
    [ret setQueue:[NSOperationQueue currentQueue]];
    
    return ret;
}

- (void)dealloc
{
    [self cancel];
    [self setQueue:NULL];
}

- (void)cancel;
{
    [[self owner] removeCompletionBlock:self];
}

@end
