//
//  FAKE_NLHTTPRequestOperation.m
//  NLDataProvider
//
//  Created by Remy Demarest on 15/11/2010.
//  Copyright 2010 NuLayer Inc. All rights reserved.
//

#import "FAKE_NLHTTPRequestOperation.h"
#import "FAKE_NSURLConnection.h"
#import "FAKE_NSHTTPURLResponse.h"

NSString *const NLOperationGETRewriteURL      = @"http://icanhascheezburger.com/rewrite";
NSString *const NLOperationGETRewrittenURL    = @"http://get.example.com/rewrite";
NSString *const NLOperationGETDelegatePrevent = @"http://get.example.com/delegateprevent";
NSString *const NLOperationGETParsedMIMEType  = @"http://get.example.com/parsedmime";
NSString *const NLOperationGETParsedURLExt    = @"http://get.example.com/parsed.ext";
NSString *const NLOperationGETParsedSuggest   = @"http://get.example.com/parsedsuggest";
NSString *const NLOperationGETRawData         = @"http://get.example.com/rawdata";

@implementation FAKE_NLHTTPRequestOperation
@synthesize isCancelled, isExecuting, isFinished, timeoutInterval, requestURL = _requestURL, requestOptions = _requestOptions;

- (id)init;
{
    [self release];
    NSAssert(NO, @"init cannot be used.");
    return nil;
}

+ (id)loadOperationWithURL:(NSURL *)aURL options:(NSDictionary *)options completion:(void (^)(NSData *resources, NSHTTPURLResponse *response, NSError *error))completion;
{
    return [[[self alloc] initWithURL:aURL options:options completion:completion] autorelease];
}

- (id)initWithURL:(NSURL *)aURL options:(NSDictionary *)options completion:(void (^)(NSData *resources, NSHTTPURLResponse *response, NSError *error))completion;
{
    if((self = [super init]))
    {
        _requestCompletionBlock = [completion copy];
        _requestURL             = [aURL copy];
        _requestOptions         = [options copy];
        
        NSString *URLString = [_requestURL absoluteString];
        
        if([URLString isEqualToString:NLOperationGETRewriteURL])
            fail(@"The data provider started a request with a URL that was supposed to be rewritten.");
        else if([URLString isEqualToString:NLOperationGETDelegatePrevent])
            fail(@"The data provider started a request that should have been prevented by the delegate.");
        else if([URLString isEqualToString:NLOperationGETRewrittenURL] ||
                [URLString isEqualToString:NLOperationGETRawData]      ||
                [URLString isEqualToString:NLOperationGETParsedURLExt])
        {
            _responseToReturn = [[FAKE_NSHTTPURLResponse alloc] init];
            [_responseToReturn setURL:_requestURL];
            [_responseToReturn setMIMEType:@"text/raw"];
            [_responseToReturn setStatusCode:200];
            [_responseToReturn setSuggestedFilename:@"suggested.raw"];
        }
        else if([URLString isEqualToString:NLOperationGETParsedMIMEType])
        {
            _responseToReturn = [[FAKE_NSHTTPURLResponse alloc] init];
            [_responseToReturn setURL:_requestURL];
            [_responseToReturn setMIMEType:@"text/ext"];
            [_responseToReturn setStatusCode:200];
            [_responseToReturn setSuggestedFilename:@"suggested.raw"];
        }
        else if([URLString isEqualToString:NLOperationGETParsedSuggest])
        {
            _responseToReturn = [[FAKE_NSHTTPURLResponse alloc] init];
            [_responseToReturn setURL:_requestURL];
            [_responseToReturn setMIMEType:@"text/raw"];
            [_responseToReturn setStatusCode:200];
            [_responseToReturn setSuggestedFilename:@"suggested.ext"];
        }
        else if([URLString isEqualToString:NLOperationGETError])
            _errorToReturn = [[NSError errorWithDomain:@"ErrorTesting" code:-1001 userInfo:nil] retain];
        else if([URLString isEqualToString:NLOperationGETCancel])
            _errorToReturn = [[NSError errorWithDomain:@"This code shouldn't be triggered" code:0 userInfo:nil] retain];
        
        _dataToReturn = [[URLString dataUsingEncoding:NSUTF8StringEncoding] copy];
    }
    return self;
}

- (void)dealloc
{
    [_requestCompletionBlock release];
    [_responseToReturn       release];
    [_dataToReturn           release];
    [_errorToReturn          release];
    [_requestURL             release];
    [_requestOptions         release];
    [super                   dealloc];
}

- (void)NL_waitForExecution
{
    [self performSelector:@selector(NL_returnResponse) withObject:nil afterDelay:0.0];
}

- (void)NL_returnResponse
{
    if([self isCancelled]) return;
    
    [self setIsFinished:YES];
    [self setIsExecuting:NO];
    
    if(_responseToReturn != nil) _requestCompletionBlock(_dataToReturn, (NSHTTPURLResponse *)_responseToReturn, nil);
    else                         _requestCompletionBlock(nil, nil, _errorToReturn);
}

- (void)start;
{
    [self setIsExecuting:YES];
    
    [self performSelector:@selector(NL_waitForExecution) withObject:nil afterDelay:0.0];
}

- (BOOL)isKindOfClass:(Class)aClass
{
    return [super isKindOfClass:aClass] || [NLHTTPRequestOperation isSubclassOfClass:aClass];
}

- (BOOL)isMemberOfClass:(Class)aClass
{
    return [super isMemberOfClass:aClass] || [NLHTTPRequestOperation class] == aClass;
}

- (void)cancel;
{
    [self setIsCancelled:YES];
    
    [self setIsFinished:YES];
    [self setIsExecuting:NO];
}

- (BOOL)isConcurrent;                       { return YES; }
- (BOOL)isReady;                            { return YES; }
- (void)main;                               { }
- (void)addDependency:(NSOperation *)op;    { }
- (void)removeDependency:(NSOperation *)op; { }
- (NSArray *)dependencies;                  { return [NSArray array]; }
@end
