//
//  FAKE_NSURLConnection.m
//  NLDataProvider
//
//  Created by Remy Demarest on 15/11/2010.
//  Copyright 2010 NuLayer Inc. All rights reserved.
//

#import "FAKE_NSURLConnection.h"
#import "FAKE_NSHTTPURLResponse.h"

NSString *const NLOperationGETRequest = @"http://get.example.com/request";
NSString *const NLOperationGETError   = @"http://get.example.com/error";
NSString *const NLOperationGET304     = @"http://get.example.com/304";
NSString *const NLOperationGET404     = @"http://get.example.com/404";
NSString *const NLOperationGETCancel  = @"http://get.example.com/cancel";
NSString *const NLOperationGETDie     = @"http://get.example.com/die";

@implementation FAKE_NSURLConnection

+ (BOOL)canHandleRequest:(NSURLRequest *)request;
{
    return [[[request URL] scheme] hasPrefix:@"http"];
}

+ (NSURLConnection *)connectionWithRequest:(NSURLRequest *)request delegate:(id)delegate;
{
    return [[[self alloc] initWithRequest:request delegate:delegate] autorelease];
}

- (id)init
{
    [self release];
    return nil;
}


- (id)initWithRequest:(NSURLRequest *)request delegate:(id)delegate;
{
    if([[[request URL] absoluteString] isEqualToString:NLOperationGETDie])
    {
        [self release];
        return nil;
    }
    
    if((self = [super init]))
    {
        _delegate = delegate;
        
        NSString *URLString = [[request URL] absoluteString];
        
        _dataToReturn = [[URLString dataUsingEncoding:NSUTF8StringEncoding] mutableCopy];
        
        if([URLString isEqualToString:NLOperationGETRequest])
        {
            _responseToReturn = [[FAKE_NSHTTPURLResponse alloc] init];
            [_responseToReturn setStatusCode:200];
            [_responseToReturn setExpectedContentLength:[_dataToReturn length]];
        }
        else if([URLString isEqualToString:NLOperationGETError])
            _errorToReturn = [[NSError errorWithDomain:@"NoConnection" code:-1001 userInfo:nil] retain];
        else if([URLString isEqualToString:NLOperationGET304])
        {
            _responseToReturn = [[FAKE_NSHTTPURLResponse alloc] init];
            [_responseToReturn setStatusCode:304];
            [_responseToReturn setExpectedContentLength:NSURLResponseUnknownLength];
            [_responseToReturn setAllHeaderFields:[NSDictionary dictionary]];
        }
        else if([URLString isEqualToString:NLOperationGET404])
        {
            _responseToReturn = [[FAKE_NSHTTPURLResponse alloc] init];
            [_responseToReturn setStatusCode:404];
            [_responseToReturn setExpectedContentLength:NSURLResponseUnknownLength];
        }
        else if([URLString isEqualToString:NLOperationGETCancel])
        {
            // TODO: setup to return GET request cancel
        }
    }
    return self;
}

- (void)dealloc
{
    [_dataToReturn release];
    [_responseToReturn release];
    [_errorToReturn release];
    [super dealloc];
}


- (id)initWithRequest:(NSURLRequest *)request delegate:(id)delegate startImmediately:(BOOL)startImmediately;
{
    FAKE_NSURLConnection *ret = [self initWithRequest:request delegate:delegate];
    [ret start];
    return ret;
}

- (void)scheduleInRunLoop:(NSRunLoop *)aRunLoop forMode:(NSString *)mode
{
    
}

- (void)unscheduleFromRunLoop:(NSRunLoop *)aRunLoop forMode:(NSString *)mode
{
    
}

- (void)NL_returnResponse
{
    if(_responseToReturn != nil)
    {
        [_delegate connection:(NSURLConnection *)self didReceiveResponse:(NSURLResponse *)_responseToReturn ];
        
        [self performSelector:@selector(NL_returnData) withObject:nil afterDelay:0.0];
    }
    else
    {
        [_delegate connection:(NSURLConnection *)self didFailWithError:_errorToReturn];
        CFRunLoopStop([[NSRunLoop currentRunLoop] getCFRunLoop]);
    }
}

- (void)NL_returnData;
{
    NSRange retRange = NSMakeRange(0, MIN(15, [_dataToReturn length]));
    
    if(retRange.length == 0)
    {
        [self performSelector:@selector(NL_returnDidFinish) withObject:nil afterDelay:0.0];
        return;
    }
    
    NSData *ret = [_dataToReturn subdataWithRange:retRange];
    
    [_delegate connection:(NSURLConnection *)self didReceiveData:ret];
    
    [_dataToReturn replaceBytesInRange:retRange withBytes:NULL length:0];
    
    [self performSelector:@selector(NL_returnData) withObject:nil afterDelay:0.0];
}

- (void)NL_returnDidFinish
{
    [_delegate connectionDidFinishLoading:(NSURLConnection *)self];
    
    CFRunLoopStop([[NSRunLoop currentRunLoop] getCFRunLoop]);
}

- (void)start;
{
    [self performSelector:@selector(NL_returnResponse) withObject:nil afterDelay:0.0];
}

- (void)cancel;
{
    _isCancelled = YES;
    CFRunLoopStop([[NSRunLoop currentRunLoop] getCFRunLoop]);
}

@end
