//
//  NLJSONDataParserTest.m
//  NLDataProvider
//
//  Created by Remy Demarest on 10/11/2010.
//  Copyright 2010 Nulayer Inc. All rights reserved.
//

#define HC_SHORTHAND

#import <Foundation/Foundation.h>
#import <NLFoundation/NLFoundation.h>
#import <Cedar/Cedar.h>
#import <OCHamcrest/OCHamcrest.h>

#import "NLJSONDataParser.h"

@interface NLJSONDataParserTest : CDRSpec @end

@implementation NLJSONDataParserTest

- (void)declareBehaviors
{
    NSData           *randomData = [@"[\"Parsed Result\"]" dataUsingEncoding:NSUTF8StringEncoding];
    NLJSONDataParser *dataParser = [[NLJSONDataParser alloc] init];
    
    NSString *result = @"Other Result";
    
    [dataParser setParseScheme:@"scheme" withParserBlock:^ id (id JSONData, NSURL *URL, NSDictionary *options, NSError **outError) { return result; }];
    
    __block id       parsedObject = nil;
    __block NSError *errorObject  = nil;
    
    describe(@"When a parser block is associated with the URL", ^{
        beforeEach(^{
            NSError *error = nil;
            parsedObject = [dataParser parsedObjectForData:randomData withURL:[NSURL URLWithString:@"scheme:cat"] options:nil error:&error];
            
            if(parsedObject == nil) errorObject = error;
        });
        
        it(@"returned objects should be the resources returned by the parser block", ^{
            assertThat(parsedObject, is(equalTo(result)));
        });
        
        it(@"error object should be nil", ^{
            assertThat(errorObject, is(equalTo(nil)));
        });
    });
    
    describe(@"When no matching parser block is found", ^{
        beforeEach(^{
            NSError *error = nil;
            parsedObject = [dataParser parsedObjectForData:randomData withURL:[NSURL URLWithString:@"path:cat"] options:nil error:&error];
            
            if(parsedObject == nil) errorObject = error;
        });
        
        it(@"returned objects should be the resources parsed from JSON", ^{
            assertThat(parsedObject, is(equalTo([NSArray arrayWithObject:@"Parsed Result"])));
        });
        
        it(@"error object should be nil", ^{
            assertThat(errorObject, is(equalTo(nil)));
        });
    });
    
    describe(@"When the JSON data is invalid", ^{
        beforeEach(^{
            NSError *error = nil;
            parsedObject = [dataParser parsedObjectForData:[@"[{]}" dataUsingEncoding:NSUTF8StringEncoding] withURL:[NSURL URLWithString:@"path:cat"] options:nil error:&error];
            
            if(parsedObject == nil) errorObject = error;
        });
        
        it(@"returned object should be nil", ^{
            assertThat(parsedObject, is(equalTo(nil)));
        });
        
        it(@"error object should be provided", ^{
            assertThat(errorObject, isNot(equalTo(nil)));
        });
    });
    
    [dataParser release];
}

@end
