//
//  NLOperationQueue.m
//  AsynchronousLoad
//
//  Created by Remy Demarest on 14/10/2010.
//  Copyright 2010 Nulayer Inc. All rights reserved.
//

#import "NLOperationQueue.h"
#import <NLFoundation/NLFoundation.h>

static void *const _NLOperationQueueObservingContext = (void *)&_NLOperationQueueObservingContext;

@implementation NLOperationQueue
@synthesize maxConcurrentOperationCount = _maxConcurrentOperationCount;

- (id)init
{
    if((self = [super init]))
    {
        _runningOperations = [[NSMutableArray alloc] init];
        _allOperations     = [[NSMutableArray alloc] init];
        
        _maxConcurrentOperationCount = -1;
    }
    return self;
}

- (void)observeOperation:(NSOperation *)op
{
    [op addObserver:self forKeyPath:@"isReady"     options:0 context:_NLOperationQueueObservingContext];
    [op addObserver:self forKeyPath:@"isFinished"  options:0 context:_NLOperationQueueObservingContext];
    [op addObserver:self forKeyPath:@"isCancelled" options:0 context:_NLOperationQueueObservingContext];
}

- (void)stopObservingOperation:(NSOperation *)op
{
    [op removeObserver:self forKeyPath:@"isReady"     context:_NLOperationQueueObservingContext];
    [op removeObserver:self forKeyPath:@"isFinished"  context:_NLOperationQueueObservingContext];
    [op removeObserver:self forKeyPath:@"isCancelled" context:_NLOperationQueueObservingContext];
}

- (void)addOperation:(NSOperation *)op;
{
    [self observeOperation:op];
    
    if(![op isCancelled]) [_allOperations addObject:op];
    
    if([op isReady] && ![op isCancelled] && (_maxConcurrentOperationCount <= 0 || [_runningOperations count] < _maxConcurrentOperationCount))
    {
        [_runningOperations addObject:op];
        [op start];
    }
}

- (NSArray *)operations;
{
    return [[_allOperations copy] autorelease];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(NSOperation *)object change:(NSDictionary *)change context:(void *)context
{
    if(context != _NLOperationQueueObservingContext)
        return [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    
    if([_runningOperations containsObject:object])
    {
        if([object isFinished])
        {
            [self stopObservingOperation:object];
            [_runningOperations removeObjectIdenticalTo:object];
            [_allOperations     removeObjectIdenticalTo:object];
        }
        
        if(_maxConcurrentOperationCount <= 0 || [_runningOperations count] < _maxConcurrentOperationCount)
        {
            object = nil;
            for(NSOperation *op in _allOperations)
                if([op isReady] && ![op isCancelled] && ![_runningOperations containsObject:op])
                {
                    object = op;
                    break;
                }
            
            if(object != nil)
            {
                [_runningOperations addObject:object];
                [object start];
            }
        }
    }
    else if([_allOperations containsObject:object])
    {
        if([keyPath isEqualToString:@"isCancelled"] || [keyPath isEqualToString:@"isFinished"])
        {
            if([object isCancelled] || [object isFinished])
            {
                [self stopObservingOperation:object];
                [_allOperations removeObjectIdenticalTo:object];
            }
        }
        else if([keyPath isEqualToString:@"isReady"] && [object isReady])
        {
            if(_maxConcurrentOperationCount <= 0 || [_runningOperations count] < _maxConcurrentOperationCount)
            {
                [_runningOperations addObject:object];
                [object start];
            }
        }
    }
}

@end
