//
//  NLReplaceClass.m
//  NLDataProvider
//
//  Created by Remy Demarest on 10/11/2010.
//  Copyright 2010 Nulayer Inc. All rights reserved.
//

#import "NLReplaceClass.h"
#import <objc/runtime.h>
#import <objc/message.h>

@interface NLReplacedClassInfo : NSObject
@property(nonatomic, assign) id (*oldAlloc)(Class self, SEL _cmd, NSZone *zone);
@property(nonatomic, assign) Class target;
@property(nonatomic, assign) Class replacement;
@end

@implementation NLReplacedClassInfo
@synthesize oldAlloc, target, replacement;
@end


static NSMutableDictionary *_NLReplacedClassMap(void)
{
    static NSMutableDictionary *replacedClassMap = nil;
    
    if(replacedClassMap == nil) replacedClassMap = [[NSMutableDictionary alloc] init];
    
    return replacedClassMap;
}

static id _NLReplace_allocWithZone_(Class self, SEL _cmd, NSZone *zone)
{
    NLReplacedClassInfo *replacer = [_NLReplacedClassMap() objectForKey:self];
    
    id ret = nil;
    
    if([replacer replacement] == self || [replacer replacement] == nil)
        ret = [replacer oldAlloc](self, _cmd, zone);
    else ret = [[replacer replacement] allocWithZone:zone];
    
    return ret;
}

static NLReplacedClassInfo *_NLReplacerForClass(Class target, BOOL createIfAbsent)
{
    NLReplacedClassInfo *ret = [_NLReplacedClassMap() objectForKey:target];
    
    if(createIfAbsent && ret == nil)
    {
        ret = [[[NLReplacedClassInfo alloc] init] autorelease];
        [ret setTarget:target];
        [_NLReplacedClassMap() setObject:ret forKey:target];
    }
    
    return ret;
}

static void _NLReplacerSetupClassForInfo(NLReplacedClassInfo *replacer)
{
    if([replacer oldAlloc] == NULL)
    {
        Method m = class_getClassMethod([replacer target], @selector(allocWithZone:));
        
        [replacer setOldAlloc:(id(*)(Class, SEL, NSZone *))method_getImplementation(m)];
        
        class_replaceMethod(((id) [replacer target])->isa, method_getName(m), (IMP)_NLReplace_allocWithZone_, method_getTypeEncoding(m));
    }
}

void NLReplaceClass(Class target, Class replacement)
{
    NLReplacedClassInfo *info = _NLReplacerForClass(target, YES);
    [info setReplacement:replacement];
    _NLReplacerSetupClassForInfo(info);
}

void NLRestoreClass(Class target)
{
    NLReplacedClassInfo *info = _NLReplacerForClass(target, NO);
    [info setReplacement:target];
}

IMP NLReplaceMethod(Class target, SEL selector, IMP replace, BOOL classMethod)
{
    if(target == Nil || selector == NULL || replace == NULL) return NULL;
    
    if(classMethod) target = ((id)target)->isa;
    
    Method m = class_getInstanceMethod(target, selector);
    
    IMP ret = NULL;
    if(m != NULL) ret = method_getImplementation(m);
    
    class_replaceMethod(target, selector, replace, method_getTypeEncoding(m));
    
    return ret;
}
