//
//  NLURLParserTest.m
//  NuDataProvider
//
//  Created by Remy Demarest on 24/11/2010.
//  Copyright 2010 NuLayer Inc. All rights reserved.
//

#define HC_SHORTHAND

#import <Foundation/Foundation.h>
#import <NLFoundation/NLFoundation.h>
#import <Cedar/Cedar.h>
#import <OCHamcrest/OCHamcrest.h>
#import "NLURLParser.h"

static NSString *const NLSchemeResult         = @"Scheme was matched";
static NSString *const NLExactPathMatchResult = @"Exact path matched";
static NSString *const NLPrefixMatchResult    = @"Prefix path matched";
static NSString *const NLSuffixMatchResult    = @"Suffix path matched";
static NSString *const NLInfixMatchResult     = @"Infix path matched";
static NSString *const NLDefaultResult        = @"No path match found";

@interface NLURLParserTest : CDRSpec
@end

@implementation NLURLParserTest
- (void)declareBehaviors
{
    describe(@"URL parser", ^{
        NLURLParser *URLParser  = [[NLURLParser alloc] initWithCopyObjects:NO];
        
        [URLParser setObject:NLSchemeResult         forURLScheme:@"scheme"          ];
        [URLParser setObject:NLExactPathMatchResult forURLPath:@"animal/cat/baghera"];
        [URLParser setObject:NLPrefixMatchResult    forURLPath:@"animal/cat/*"      ];
        [URLParser setObject:NLSuffixMatchResult    forURLPath:@"*/lolcat"          ];
        [URLParser setObject:NLInfixMatchResult     forURLPath:@"*/cat/*"           ];
        [URLParser setDefaultObject:NLDefaultResult];
        
        it(@"should match the scheme", ^{
            assertThat([URLParser objectForURL:[NSURL URLWithString:@"scheme:animal/cat/baghera"]], is(equalTo(NLSchemeResult)));
        });
        
        it(@"should match the exact path", ^{
            assertThat([URLParser objectForURL:[NSURL URLWithString:@"path:animal/cat/baghera"]], is(equalTo(NLExactPathMatchResult)));
        });
        
        it(@"should match the path prefix", ^{
            assertThat([URLParser objectForURL:[NSURL URLWithString:@"path:animal/cat/ashoka"]], is(equalTo(NLPrefixMatchResult)));
        });
        
        it(@"should match the path suffix", ^{
            assertThat([URLParser objectForURL:[NSURL URLWithString:@"path:meme/lolcat"]], is(equalTo(NLSuffixMatchResult)));
        });
        
        it(@"should match the path infix", ^{
            assertThat([URLParser objectForURL:[NSURL URLWithString:@"path:meme/cat/ashoka"]], is(equalTo(NLInfixMatchResult)));
        });
        
        it(@"should return the default parser", ^{
            assertThat([URLParser objectForURL:[NSURL URLWithString:@"path:animal/dog/husky"]], is(equalTo(NLDefaultResult)));
        });
        
        [URLParser release];
    });
}
@end
